# -*- coding: utf-8 -*-
"""
Функции загрузки и сохранения rdm файлов

(c)2012 ЗАО "РИПАС"
www.ripas.ru
"""
import sys,os
from RoadCentre import project
from gui import imageView, msgError
from PythonQt import QtCore, QtGui

def relativePath(path,prj_dir):
    try:
        return os.path.normpath(os.path.relpath(path,prj_dir))
    except:
        return path

def parseImagePath(project_dir,schema_file,def_ext='.svg'):
    if not schema_file:
        return schema_file
    schema_file=parseV2String(schema_file).replace('\\','/')
    if not os.path.isabs(schema_file):
        schema_file=os.path.join(project_dir,schema_file)
    if not os.path.exists(schema_file):
        schema_file=os.path.splitext(schema_file)[0]+def_ext
    return schema_file

def rdmV2FloatString(v):
    return '%.12f'%v

def parseV2String(v):
    if v.__class__==tuple().__class__:
        v=','.join(v)
    return v

def loadRdm(path):
    project_dir=os.path.dirname(path)
    rdm=QtCore.QSettings(path,QtCore.QSettings.IniFormat)

    rdm.setIniCodec('UTF-8')
    rdm.beginGroup('Objects')
    objects_count=int(rdm.value('Count',0))
    for i in range(objects_count):
        #if not i%10:
        #	QtGui.QApplication.instance().processEvents()
        rdm.beginGroup(str(i))
        light=project.createObject()
        try:
            light.id=int(rdm.value('ID'))
        except:
            msgError("File %s, section 'Objects\\%d'. Value 'ID' is missing"%(rdm.fileName(), i))
            return False
        child_groups=rdm.childGroups()

        light.type=rdm.value('Type')
        light.description=parseV2String(rdm.value('Description'))
        for k,v in rdm.value('CustomProperties',{}).items():
            light.setCustomProperty(k,v[0],v[1])
        light.eventMask=int(rdm.value('EventMask/Display'))|(1<<14)
        light.position=QtCore.QPointF(float(rdm.value('PositionX')),float(rdm.value('PositionY')))
        light.setProperty('shema',parseImagePath(project_dir,rdm.value('Shema', '')))
        light.setProperty('RcdControl',rdm.value('RcdControl',0))

        if 'Phases' in child_groups:
            rdm.beginGroup('Phases')
            for p in range(int(rdm.value('Count',0))):
                rdm.beginGroup(str(p))
                new_phase=light.createPhase(int(rdm.value('ID')),parseV2String(rdm.value('Description')))
                new_phase.setProperty('shema',parseImagePath(project_dir,rdm.value('Shema','')))
                rdm.endGroup()
            rdm.endGroup()

        if 'Program' in child_groups:
            rdm.beginGroup('Program')
            for p in range(int(rdm.value('Count',0))):
                rdm.beginGroup(str(p))
                new_program=light.createProgram(int(rdm.value('ID')),parseV2String(rdm.value('Description')))
                rdm.endGroup()
            rdm.endGroup()

        project.addObject(light)
        rdm.endGroup()
    rdm.endGroup()

    if imageView!=None:
        rdm.beginGroup('Map')
        try:
            imageView.imagePath=parseImagePath(project_dir,rdm.value('FileName'),'.png')
            imageView.binding='%.12f %.12f %.12f %.12f'%(float(rdm.value('TopLeftX')),float(rdm.value('TopLeftY')),float(rdm.value('BottomRightX')),float(rdm.value('BottomRightY')))
        except:
            imageView.clear()
        rdm.endGroup()

    rdm.beginGroup('Connection')
    project.connection.remoteHost=rdm.value("ServerName","localhost")
    project.connection.remotePort=int(rdm.value("ServerPort","9000"))
    rdm.endGroup()

    from spectr import sp_module
    rdm.beginGroup('SysEventsMask')
    filter_text=rdm.value("consoleRequestFilter",None)
    if filter_text:
        project.setProperty("consoleRequestFilter",filter_text)
        sp_module.requestFilter=filter_text

    filter_text=rdm.value("consoleResponseFilter",None)
    if filter_text:
        project.setProperty("consoleResponseFilter",filter_text)
        sp_module.responseFilter=filter_text

    filter_text=rdm.value("consoleNotificationFilter",None)
    if filter_text:
        project.setProperty("consoleNotificationFilter",filter_text)
        sp_module.notificationFilter=filter_text
    rdm.endGroup()

    project.fileName=path
    project.title=rdm.value('Description/Title',path)
    project.baseDir=project_dir
    project.projectLoaded(u'Проект %s загружен'%project.title)
    return True

def saveRdm(path):
    project_dir=os.path.dirname(project.fileName)
    from tempfile import NamedTemporaryFile
    tf=NamedTemporaryFile(delete=False)
    tf.close()
    rdm=QtCore.QSettings(tf.name,QtCore.QSettings.IniFormat)
    rdm.setIniCodec('UTF-8')

    rdm.beginGroup('Description')
    rdm.setValue("Type","RoadMap configuration file")
    rdm.setValue("Version","2.00")
    rdm.setValue("Title",project.title)
    rdm.endGroup()

    rdm.beginGroup('SysEventsMask')
    rdm.setValue("Journal","0")
    rdm.setValue("Console","0")
    rdm.setValue("Display","63")
    filter_text=project.property("consoleRequestFilter")
    if filter_text: rdm.setValue("consoleRequestFilter",filter_text)
    filter_text=project.property("consoleResponseFilter")
    if filter_text: rdm.setValue("consoleResponseFilter",filter_text)
    filter_text=project.property("consoleNotificationFilter")
    if filter_text: rdm.setValue("consoleNotificationFilter",filter_text)
    rdm.endGroup()


    if imageView!=None and len(imageView.imagePath)>0:
        rdm.beginGroup("Map")
        rect=imageView.geoRect
        rdm.setValue("TopLeftX",rdmV2FloatString(rect.left()))
        rdm.setValue("BottomLeftX",rdmV2FloatString(rect.left()))
        rdm.setValue("TopLeftY",rdmV2FloatString(rect.top()))
        rdm.setValue("TopRightY",rdmV2FloatString(rect.top()))
        rdm.setValue("BottomRightX",rdmV2FloatString(rect.right()))
        rdm.setValue("TopRightX",rdmV2FloatString(rect.right()))
        rdm.setValue("BottomRightY",rdmV2FloatString(rect.bottom()))
        rdm.setValue("BottomLeftY",rdmV2FloatString(rect.bottom()))
        rdm.setValue("FileName",relativePath(imageView.imagePath,project_dir))
        rdm.setValue("Attached","1")
        rdm.endGroup()

    rdm.beginGroup("Connection")
    rdm.setValue("ServerName",project.connection.remoteHost)
    rdm.setValue("ServerPort",project.connection.remotePort)
    rdm.setValue("AutoConnect",1)
    rdm.setValue("ConnectTryMax",99)
    rdm.setValue("ReconnectTimeOut",3)
    rdm.endGroup()

    rdm.beginGroup('Objects')
    obj_count=0
    for obj in project.objects():
        if hasattr(obj,"temporary") and obj.temporary==True:
            continue
        section_name='%d/' % obj_count
        rdm.setValue(section_name+"Type",obj.type)
        rdm.setValue(section_name+"ID",obj.id)
        rdm.setValue(section_name+"Description",obj.description)
        rdm.setValue(section_name+"CustomProperties",obj.customProperties())
        if hasattr(obj,'shema'):
            rdm.setValue(section_name+"Shema",relativePath(obj.shema,project_dir))
        rdm.setValue(section_name+"PositionX",rdmV2FloatString(obj.position.x()))
        rdm.setValue(section_name+"PositionY",rdmV2FloatString(obj.position.y()))
        rdm.setValue(section_name+"RcdControl",obj.RcdControl)

        event_section=section_name+'EventMask/'
        rdm.setValue(event_section+"Journal","0")
        rdm.setValue(event_section+"Console","3")
        rdm.setValue(event_section+"Display",obj.eventMask&(~(1<<14)))

        obj_count=obj_count+1

        phase_count=0
        phase_section=section_name+"Phases/"
        for phase in obj.phases():
            subsection=phase_section+"%d/"%phase_count
            rdm.setValue(subsection+"ID",phase.id)
            rdm.setValue(subsection+"Description",phase.description)
            schema_path=relativePath(phase.property('shema'),project_dir)
            if schema_path:
                rdm.setValue(subsection+"Shema",schema_path)
            phase_count=phase_count+1
        rdm.setValue(phase_section+"Count",phase_count)

        program_count=0;
        program_section=section_name+"Program/"
        for program in obj.programs():
            subsection=program_section+"%d/"%program_count
            rdm.setValue(subsection+"ID",program.id)
            rdm.setValue(subsection+"Description",program.description)
            program_count=program_count+1
        rdm.setValue(program_section+"Count",program_count)

    rdm.setValue("Count",obj_count)
    rdm.endGroup()
    rdm.sync()
    if rdm.status() != QtCore.QSettings.NoError:
        msgError("QSettings sync error #%d" % rdm.status())
        QtCore.QFile(tf.name).remove()
        return False
    old_rdm_base_name=os.path.splitext(path)[0]
    if os.path.exists(path):
        bak_file=old_rdm_base_name+'.bak'
        if os.path.exists(bak_file):
            QtCore.QFile(bak_file).remove()
        QtCore.QFile.copy(path,bak_file)
        QtCore.QFile(path).remove()
    QtCore.QFile(tf.name).rename(path)
    return True

import actions
actions.openFileDialog.nameFilter[u"*.rdm2"] = (u"Файлы проекта Road Centre 2.0", loadRdm, True)
actions.saveFileDialog.nameFilter[u"*.rdm2"] = (u"Файлы проекта Road Centre 2.0", saveRdm)
del actions
