# -*- coding: utf-8 -*-
"""
(c)2011-2012 ЗАО "РИПАС" и Демин Федор,
www.ripas.ru dff@ripas.ru
"""
"""
Обработчики событий проекта, сoединения, устройств
"""

from PythonQt import Qt, QtCore
import gui
from gui import mainWindow, widgetFromUi
import RoadCentre
from RoadCentre import kernel, plugins, project
import os
import wallet


def execDirScripts(dir_name, base_dir=None):
    if base_dir:
        import sys
        sys.path.append(base_dir)
        full_dir_path = os.path.join(base_dir, dir_name)
    else:
        full_dir_path = os.path.join(os.getcwd(), dir_name)
    import runpy
    for dirpath, dirnames, filenames in os.walk(full_dir_path):
        filenames.sort()
        for name in filenames:
            if os.path.splitext(name)[-1] != '.py': continue
            # execfile(os.path.join(dirpath, name))
            runpy.run_path(os.path.join(dirpath, name))
    if base_dir:
        sys.path.pop()


def showStatus():
    try:
        total_count = len(project.objects())
        connected_count = 0
        for o in project.objects():
            if o.isConnected:
                connected_count+=1
        mainWindow.statusBar().findChild('QLabel').setText(u'Подключено %d из %d' % (connected_count,total_count))
    except:
        pass


# 0x1100:GET_STAT, 0x100b:SET_EVENT, 0x110c:GET_VPU
requestFilter = 'request.command not in (0x1100, 0x100b, 0x110c)'


def objectRequest(request, obj):
    if objectRequest.filter != requestFilter:
        objectRequest.filterCode = compile(requestFilter, 'requestFilter', 'eval')
        objectRequest.filter = requestFilter

    obj_console = getattr(obj, 'console', None)
    if obj_console:
        local_filter = getattr(obj, 'consoleRequestFilter', None)
        if local_filter and eval(local_filter):
            gui.msgInfo(u'Отправка команды:\n', obj_console)
            gui.msgAdd(request.toString()+'\n', 0, obj_console)

    if not eval(objectRequest.filterCode): return
    gui.msgInfo(u'Объект №%d(%s). Отправка команды:\n' % (obj.id, obj.description))
    gui.msgAdd(request.toString()+'\n')


objectRequest.filter = None

# 0x1100:GET_STAT, 0x100b:SET_EVENT, 0x110c:GET_VPU
responseFilter = 'resp.request.command not in (0x1100, 0x100b, 0x110c) or resp.error not in (0, 0x1200)'


def objectResponseFinished(resp, obj):
    if objectResponseFinished.filter != responseFilter:
        objectResponseFinished.filterCode = compile(responseFilter, 'responseFilter', 'eval')
        objectResponseFinished.filter = responseFilter

    error = resp.error
    if error in (0, 0x1200):
        printFun = gui.msgOk
    else:
        printFun = gui.msgError

    obj_console = getattr(obj, 'console', None)
    if obj_console:
        local_filter = getattr(obj, 'consoleResponseFilter', None)
        if local_filter and eval(local_filter):
            printFun(u'Ответ на команду:\n', obj_console)
            gui.msgAdd(resp.request.toString()+'\n', 0, obj_console)
            err = error
            if err == 0x1208:  # заплатка, учитывающая код ошибки NOT_EXEC
                try:
                    err += resp.rawData.at(16) - 1
                except:
                    pass
                gui.msgAdd(u'Результат: %s.\n' % (resp.errorString(err, True)), 0, obj_console)
            else:
                gui.msgAdd(u'Результат: %s %s.\n' % (resp.errorString(True), resp.toString()), 0, obj_console)
            if resp.timeStamp.isValid():
                gui.msgAdd(u'Штамп времени: %s.\n' % resp.timeStamp.time(), 0, obj_console)

    if not eval(objectResponseFinished.filterCode): return
    printFun(u'Объект №%d(%s). Ответ на команду:\n' % (obj.id, obj.description))
    gui.msgAdd(resp.request.toString()+'\n')
    if error == 0x1208:  # заплатка, учитывающая код ошибки NOT_EXEC
        try:
            error += resp.rawData.at(16) - 1
        except:
            pass
        gui.msgAdd(u'Результат: %s.\n' % (resp.errorString(error, True)))
    else:
        gui.msgAdd(u'Результат: %s %s.\n' % (resp.errorString(True), resp.toString()))
    if resp.timeStamp.isValid():
        gui.msgAdd(u'Штамп времени: %s.\n' % resp.timeStamp.time())


objectResponseFinished.filter = None

notificationFilter = 'notify.id not in (4,)'


def objectNotification(notify, obj):
    if objectNotification.filter != notificationFilter:
        objectNotification.filterCode = compile(notificationFilter, 'notificationFilter', 'eval')
        objectNotification.filter = notificationFilter

    if notify.id in (1, 14):
        printFun = gui.msgError
    elif notify.id == 2:
        printFun = lambda msg, alt_console=None: gui.msgNew(msg, QtCore.Qt.darkYellow, alt_console)
    else:
        printFun = gui.msgOk

    obj_console = getattr(obj, 'console', None)
    if obj_console:
        local_filter = getattr(obj, 'consoleNotificationFilter', None)
        if local_filter and eval(local_filter):
            printFun(u'Сообщение:\n', obj_console)
            gui.msgAdd(notify.toString()+'\n', 0, obj_console)
            if notify.timeStamp.isValid():
                gui.msgAdd(u'Штамп времени: %s.\n' % notify.timeStamp.time(), 0, obj_console)

    if not eval(objectNotification.filterCode): return
    printFun(u'Объект №%d(%s). Сообщение:\n' % (obj.id, obj.description))
    gui.msgAdd(notify.toString()+'\n')
    if notify.timeStamp.isValid():
        gui.msgAdd(u'Штамп времени: %s.\n' % notify.timeStamp.time())


objectNotification.filter = None

# connectedFilter='1'
# Фильтр, отсеиваюший массовые сообщения при изменении состояния подключения к серверу:
connectedFilter = 'project.connection.state==4 and (not isConnected or (abs(obj.connectionTime.secsTo(QtCore.QDateTime.currentDateTime())) < 60))'
"""
При отключении от сервера отфильтровывается все по connection.state.
При подключении выводятся только объекты, с connectionTime в пределах 60 секунд от текущего времени.
Примечание: если время сервера и компьютера сильно различаются (в данном случае более, чем 60 сек),
            этот код будет работать некорректно
"""


def objectConnected(isConnected, obj):
    showStatus()
    if objectConnected.filter != connectedFilter:
        objectConnected.filterCode = compile(connectedFilter, 'connectedFilter', 'eval')
        objectConnected.filter = connectedFilter
    if not eval(objectConnected.filterCode): return
    if isConnected:
        gui.msgOk(u'Объект №%d(%s) подключение' % (obj.id, obj.description))
    else:
        gui.msgNew(u'Объект №%d(%s) отключение' % (obj.id, obj.description), QtCore.Qt.darkYellow)


objectConnected.filter = None


def projectLoaded(msg):
    project.changed = False
    gui.msgOk(msg)
    execDirScripts('popen', os.path.join(project.baseDir, 'scripts'))
    execDirScripts('popen')


def projectClosed(msg):
    from presets import clearPresets
    mainWindow.windowTitle = project.title
    gui.msgInfo(msg)
    kernel.selectionModel.clearSelection()
    execDirScripts('pclose')
    execDirScripts('pclose', os.path.join(project.baseDir, 'scripts'))
    clearPresets()
    gui.osmView.imagePath = ''
    if gui.osmView2: gui.osmView.imagePath = ''
    reconnect.timer.stop()
    project.connection.stop()


def reconnect():
    connection = project.connection
    if connection.state == connection.UnconnectedState:
        gui.msgInfo(u'Восстановление подключения...')
        project.connection.start()


reconnect.timer = QtCore.QTimer()
reconnect.timer.connect('timeout()', reconnect)

def connectionStateChanged(state):
    def intrernal_parse():
        parseConnectionState(state)
    if state == project.connection.UnconnectedState and not getattr(project.connection, 'manualDisconnect', False):
        QtCore.QTimer.singleShot(1000,intrernal_parse)
    else:
        parseConnectionState(state)

def parseConnectionState(state):
    from settings import userSettings
    connection = project.connection
    action = mainWindow.findChild(Qt.QAction, 'actionConnect')
    if action: action.checked = connection.state > connection.UnconnectedState
    if state in (connection.UnconnectedState, connection.ConnectingState,
                 connection.IdentificationState, connection.AuthenticationState):
        fun = gui.msgInfo
    elif state in (connection.ConnectedState, connection.ReadyState):
        fun = gui.msgOk
    elif state == connection.AuthErrorState:
        fun = gui.msgError
    else:
        fun = gui.msgWarning
    fun(u'Сервер %s:%u: %s' % (connection.remoteHost, connection.remotePort, connection.stateString(state)))

    if state <= connection.UnconnectedState:
        if project.fileName and \
                (not hasattr(connection, 'manualDisconnect') or not connection.manualDisconnect):
            reconnect.timer.start(1000)
    else:
        reconnect.timer.stop()

    if state == connection.ReadyState:
        userName = connection.userName
        if userName:
            gui.msgAdd(u' %s' % userName, QtCore.Qt.darkGreen)
            users = userSettings.value('Users')
            if not users: users = ()
            if users.__class__ != tuple: users = (users,)
            if userName not in users:
                users = users + (userName,)
                userSettings.setValue('Users', users)

    if state in (connection.AuthRequiredState, connection.AuthErrorState):
        if bool(connection.userName) ^ (state == connection.AuthErrorState):
            if connection.userName: connection.authenticate()
        else:
            auth_wnd = gui.widgetFromUi('spectr/authdlg.ui')
            #Имя используется в системе обновлений для проверки активно-ли сейчас окно аутентификации
            auth_wnd.setObjectName('auth_wnd')
            useWallet = auth_wnd.findChild('QCheckBox', 'useWallet')
            if useWallet:
                useWallet.visible = wallet.hasWallet()
                useWallet.checked = userSettings.value('use_wallet', False)

            auth_wnd.findChild('QLabel', 'realm').text = u'Аутентификация на %s:%u' % (
            connection.remoteHost, connection.remotePort)
            user = auth_wnd.findChild('QComboBox', 'user')
            password = auth_wnd.findChild('QLineEdit', 'password')
            user.addItems(userSettings.value('Users', ()))
            user.currentIndex = -1

            def user_name_changed(idx):
                user_name = user.itemText(idx)
                if not useWallet or not wallet.hasWallet() or not useWallet.checked:
                    return
                password.text = wallet.getPass(userSettings.applicationName(), user_name)

            if wallet.hasWallet():
                user.connect('currentIndexChanged(int)', user_name_changed)
            prj_hash = QtCore.QCryptographicHash.hash(project.fileName,QtCore.QCryptographicHash.Md4).toHex().data().decode()
            prj_user_key = 'ProjectUsers/'+prj_hash
            cur_uname = userSettings.value(prj_user_key)
            if cur_uname is None or len(cur_uname) == 0:
                cur_uname = userSettings.value('last_user', connection.userName)
            if len(cur_uname):
                u_idx = user.findText(cur_uname)
                if u_idx < 0:
                    user.addItem(cur_uname)
                    u_idx = user.count - 1
                user.currentIndex = u_idx
            else:
                user.currentIndex = 0
            try:
                if getattr(auth_wnd, 'exec')():
                    connection.authenticate(user.currentText, password.text)
                    if hasattr(kernel,'wallet'):
                        kernel.wallet.unlock(user.currentText, password.text)
                    if wallet.hasWallet() and useWallet:
                        userSettings.setValue('use_wallet', useWallet.checked)
                        userSettings.setValue('last_user', user.currentText)
                        userSettings.setValue(prj_user_key, user.currentText)
                        wallet.savePass(userSettings.applicationName(), user.currentText, password.text)
                    return
            finally:
                auth_wnd.deleteLater()
            # Отключится, если пользователь прервал аутентификацию
            connection.setProperty('manualDisconnect', True)
            connection.stop()
    showStatus()


def relogin(user='', password=''):
    project.connection.stop()
    project.connection.authenticate(user, password)
    project.connection.start()


def connectToProject(prj):
    global project
    project = prj
    project.connect('projectLoaded(const QString&)', projectLoaded)
    project.connect('projectClosed(const QString&)', projectClosed)

    project.connect('objectRequest(const QObject*, const Spectr::Light*)', objectRequest)
    project.connect('objectResponseFinished(const QObject*, const Spectr::Light*)', objectResponseFinished)
    project.connect('objectNotification(const QObject*, const Spectr::Light*)', objectNotification)
    project.connect('objectConnected(bool, const Spectr::Light*)', objectConnected)

    project.connection.connect('stateChanged(const Spectr::Connection::State)', connectionStateChanged)
