# -*- coding: utf-8 -*-
"""
(c)2011-2012 ЗАО "РИПАС" и Демин Федор,
www.ripas.ru dff@ripas.ru
"""
"""
Формирует меню объекта контроллера "СПЕКТР"
"""

from PythonQt import Qt, QtCore, QtGui
from gui import mainWindow, osmView, showSelectionOnTree
from RoadCentre import plugins
from RoadCentre import project,kernel
if hasattr(plugins,'TimelinePlugin'):
    from timeline import findSynchronizerForTimeline
from actions import showControllerOnMap
from menu import SubMenu
from spectr import getDeviceVersion
try:
    import metarpc
    has_metarpc_module = True
except ModuleNotFoundError:
    has_metarpc_module = False

class ControllerMenu(SubMenu):
    def __init__(self):
        SubMenu.__init__(self, (u'Контроллер', u'Контроллеры'))
        self.__actions = ()
        gr = Qt.QActionGroup(None)
        self.__groupCmd = gr
        gr.connect('triggered(QAction*)', self.actionDeviceCommand)

        self.menuPhase = Qt.QMenu(u'Установить фазу')
        self.menuProgram = Qt.QMenu(u'Установить программу')

        m = Qt.QMenu(u'Специальные')
        self.menuSpecial = m
        command = (
            (u'ЖМ', ('SET_YF', ''), u'Установить режим желтого мигания'),
            (u'ОС', ('SET_OS', ''), u'Отключить светофор'),
            (u'Рестарт', ('SET_START', ''), u'Перезапуск контроллера'),
            )
        for cmd in command:
            a = m.addAction(cmd[0])
            a.setData(cmd[1])
            a.toolTip = cmd[2]
            a.statusTip = cmd[2]
            gr.addAction(a)

        m = Qt.QMenu(u'Служебные')
        self.menuService = m
        if hasattr(plugins,'DownloadQueue'):
            a = m.addAction(u'Считать конфигурацию')
            a.setData('GET_CONFIG')
            a.connect('triggered()', self.getConfig)
            self.mfsa = m.addAction(u'Считать конфигурацию из хранилища')
            self.mfsa.connect('triggered()', self.getConfigFromStorage)
            if self.__has_metarpc():
                self.mfsa.setData('MetaRpc/INVOKE_StorageApp_getConfig(QString)')

        if hasattr(plugins,'concon') and hasattr(plugins,'DownloadQueue'):
            self.mopenconf = m.addAction(u'Открыть сохранённую конфигурацию')
            self.mopenconf.connect('triggered()', self.openSavedConfig)

        a = m.addAction(u'Электропитание...')
        a.setData('GET_POWER')
        a.connect('triggered()', self.onPowerDlg)
        self.__powerDlg = a

        a = m.addAction(u'Установить время системы')
        a.connect('triggered()', self.setSystemTime)
        a.setData(('SET_DATE','SET_TIME'))
        
        self.updateFirmwareAction = None
        if hasattr(plugins,'Service'):
            self.gevInfoAction = m.addAction(u'Информация об устройстве')
            self.gevInfoAction.connect('triggered()', self.getDeviceInfo)
            self.gevInfoAction.setData(('GET_DEVICE','GET_CLIST'))
            a = m.addAction(u'Обновить конфигурацию')
            a.setData('SET_CONFIG')
            a.connect('triggered()', self.updateConfig)
            self.updateFirmwareAction = m.addAction(u'Обновить внутреннее ПО')
            self.updateFirmwareAction.connect('triggered()', self.updateFirmware)
            self.updateFirmwareAction.enabled = False
            from service import objectConsole
            a = m.addAction(u'Консоль')
            a.setData('URes/DEV_CONSOLE')
            a.connect('triggered()', objectConsole)
        if hasattr(plugins,'TimelinePlugin'):
            self.sgDiagramm=m.addAction(u'Диаграмма сигнальных групп')
            self.sgDiagramm.connect('triggered()', self.showSgDiagramm)
            self.sgDiagramm.visible=False


        command = (
            (u'Локальный режим', ('SET_LOCAL', ''), u'Перевести контроллер в локальный режим (отменить управление АСУДД)'),
            (u'Обновить статус', ('GET_STAT', ''), u'Обновить статус'),
            )
        for cmd in command:
            a = Qt.QAction(cmd[0], None)
            a.setData(cmd[1])
            a.toolTip = cmd[2]
            a.statusTip = cmd[2]
            gr.addAction(a)
            self.__actions = self.__actions + (a,)

        a = Qt.QAction(u'Очистить статистику', None)
        a.connect('triggered()', self.clearStatistics)
        self.__actions = self.__actions + (a,)

        self.groupJourn = Qt.QActionGroup(None)
        grJourn = self.groupJourn
        grJourn.connect('triggered(QAction*)', self.getJournal)
        m = Qt.QMenu(u'Журнал')
        self.menuJourn = m
        command = (
            (u'Скачать полностью', -1),
            (u'Открыть', -2),
            (u'Докачать', -3),
            (u'1 блок', 1),
            (u'5 блоков', 5),
            (u'10 блоков', 10),
            )
        for cmd in command:
            a = m.addAction(cmd[0])
            a.setData(cmd[1])
            # a.toolTip=cmd[2]
            # a.statusTip=cmd[2]
            grJourn.addAction(a)

        m = Qt.QMenu(u'ВПУ')
        # menu.addMenu(m)
        self.menuEcu = m
        command = (
            (u'Состояние', ('GET_VPU', ''), u'Опросить состояние ВПУ'),
            (u'Разрешить', ('SET_VPU', 0), u'Разрешить управление от ВПУ'),
            (u'Заблокировать', ('SET_VPU', 1), u'Блокировка управления от ВПУ'),
            )
        for cmd in command:
            a = m.addAction(cmd[0])
            a.setData(cmd[1])
            a.toolTip = cmd[2]
            a.statusTip = cmd[2]
            gr.addAction(a)

    def changeProg(self):
        if not self.__has_cnfstorage():
            return
        for dev in self.__connected:
            plugins.concon.editDprog(dev.id,dev.program)

    def cancelProg(self):
        if not self.__has_cnfstorage():
            return
        for dev in self.__connected:
            self._cancel_dev_prog(dev)

    def actionDeviceCommand(self, act):
        data = act.data()
        cmd = data[0]
        param = data[1]
        for dev in self.__connected:
            if cmd == 'SET_LOCAL' and dev.currentProgram > 49 and dev.currentProgram < 52:
                self._cancel_dev_prog(dev)
            dev.sendCommand(cmd, param)

    def setSystemTime(self):
        import datetime
        for dev in self.__connected:
            dt = datetime.datetime.now()  # Для повышения точности запрашиваем время каждый раз, хотя, наверно, можно безболезненно вынести за пределы цикла
            dev.sendCommand('SET_TIME', dt.strftime('%H:%M:%S'))
            dev.sendCommand('SET_DATE', dt.strftime('%d/%m/%y'))

    def clearStatistics(self):
        for dev in self.selection: dev.statistics.clear()

    def project(self):
        if not plugins.SpectrPlugin: return None
        for ch in plugins.SpectrPlugin.children():
            if kernel.objectClassName(ch) == 'Spectr::Project':
                return ch
        return None

    def getConfig(self):
        for dev in self.__connected:
            plugins.DownloadQueue.loadConfig(dev)

    def getDeviceInfo(self):
        for dev in self.__connected:
            if dev.type[:6] == u'Спектр' or dev.type[:7] == u'UK_GEAR':
                plugins.Service.showDeviceInfo(dev)

    def updateConfig(self):
        for dev in self.__connected:
            plugins.Service.uploadConfig(dev)

    def updateFirmware(self):
        for dev in self.__connected:
            plugins.Service.updateFirmware(dev)

    def getJournal(self, act):
        blocks = act.data()
        if not blocks:
            blocks=-1
        if blocks==-2:
            d = QtCore.QDir(plugins.DownloadQueue.journalDir)
            a = self.menuJourn.actions()
            for dev in self.selection:
                journal_name='Jrn%05u.bin' % dev.id
                if d.exists(journal_name):
                    plugins.DownloadQueue.viewJournal(getDeviceVersion(dev),d.absoluteFilePath(journal_name))
            return
        for dev in self.__connected:
            plugins.DownloadQueue.loadJournal(dev,blocks)

    def onPowerDlg(self):
        import powerControl
        for o in self.__connected:
            if o.unitFlags & 0x300: powerControl.PowerControlDialog(o, mainWindow)

    def controlPanel(self):
        for o in self.selection:
            panels = plugins.ControlPanelPlugin.panels()
            d = plugins.ControlPanelPlugin.open(o)
            if not mainWindow.restoreDockWidget(d):
                for w in panels:
                    if not w.isFloating():
                        mainWindow.tabifyDockWidget(w, d)
                        break
                else: mainWindow.addDockWidget(4, d)
            d.show()
            getattr(d, 'raise')()

    def getConfigFromStorage(self):
        if not self.__has_cnfstorage():
            return
        for dev in self.selection:
            if self.__has_metarpc():
                rpc_module = metarpc.cnfStorage.rpc
            else:
                rpc_module = plugins.JsonRpcClientPlugin
            plugins.DownloadQueue.loadConfigFromStorage(rpc_module,dev)

    def openSavedConfig(self):
        for dev in self.selection:
            if hasattr(dev,'dqueue_config'):
                plugins.concon.openConfig(dev.dqueue_config)

    def showSgDiagramm(self):
        if not len(self.selection):
            return
        doc_window=plugins.TimelinePlugin.createObjectTimelineWidgetSg(mainWindow,self.selection[0])
        w=doc_window.findChild('TimelineWidget')
        if w:
            findSynchronizerForTimeline(w)
        from timeline import docTimelineWidget
        docTimelineWidget(doc_window)

    def beforeMenu(self,menu,connected):
        pass

    def menu(self, obj):
        if hasattr(project.connection,'commandAvailable'):
            commandAvailable=lambda cmd,prot='Spectr': project.connection.commandAvailable(prot,cmd)
        else:
            commandAvailable=lambda cmd,prot='Spectr': True
        menu = self.menuMain
        menu.clear()

        connected = []  # Список подключенных
        spectr = []  # Список Спектров
        uk_gear = []  # Список ukgear
        spectr2 = []
        mfu = []
        ecuEnabled = True  # Меню ВПУ
        self.__powerDlg.visible = False
        if hasattr(self,'mfsa'): self.mfsa.enabled=False
        for o in obj:
            if o.isConnected:
                connected.append(o)
                if o.ecuStatus < 0 and (not hasattr(o,'RcdControl') or o.RcdControl==0): ecuEnabled = False
                if o.unitFlags & 0x300: self.__powerDlg.visible = True
                if self.__has_cnfstorage():
                    if hasattr(self,'mfsa') and not self.mfsa.enabled:
                        self.mfsa.enabled = True
            if o.type[:6] == u'Спектр': spectr.append(o)
            if o.type[:7] == u'UK_GEAR': uk_gear.append(o)
            if o.type[:7].lower() == u'spectr2': spectr2.append(o)
            if o.type[:3].lower() == u'мфу': mfu.append(o)
        self.__connected = connected
        self.__spectr = spectr
        count = len(connected)
        spCount = len(spectr)
        sp2Count = len(spectr2)
        ukgCount = len(uk_gear)
        mfuCount = len(mfu)
        if self.updateFirmwareAction:
            self.updateFirmwareAction.enabled = spCount>0
            
        if count == 0 and self.__has_cnfstorage():
            if hasattr(self, 'mfsa'):
                self.mfsa.enabled = True
                menu.addAction(self.mfsa)
            if hasattr(self, 'mopenconf'):
                menu.addAction(self.mopenconf)
        if hasattr(self,'cnfAchiveMenu'):
            self.cnfAchiveMenu.delete()
            del self.cnfAchiveMenu
        if self.__has_jsonrpc_cnfstorage() and len(obj)==1:
            oid = obj[0].id
            ans = plugins.JsonRpcClientPlugin.sendCommand('cnfStorage','app.getConfigHistorySize',[oid])
            history_size = 0
            cnf_dates = ()
            if ans:
                history_size = ans.get('result',0)
            if history_size:
                ans = plugins.JsonRpcClientPlugin.sendCommand('cnfStorage','app.getConfigHistory',[oid,0,10])
                if ans:
                    cnf_dates = ans.get('result',())
            if len(cnf_dates) > 0:
                parent_menu = self.menuService if count>0 else menu
                self.cnfAchiveMenu=parent_menu.addMenu('История конфигураций')
                if hasattr(self,'mfsa'):
                    parent_menu.insertMenu(self.mfsa,self.cnfAchiveMenu)
                else:
                    parent_menu.addMenu(self.cnfAchiveMenu)
                gr = Qt.QActionGroup(parent_menu)
                for d in cnf_dates:
                    gr.addAction(self.cnfAchiveMenu.addAction(d))
                gr.connect('triggered(QAction*)', self.actionReadCnfHistory)
                # if history_size > len(cnf_dates):
                #     a = self.cnfAchiveMenu.addAction('Ещё ...')
                #     a.connect('triggered()', self.actionCnfFullHistory)

        if hasattr(self,'gevInfoAction'):
            self.gevInfoAction.enabled=(spCount!=0 or ukgCount!=0 or sp2Count!=0)

        if count:
            if count == 1:
                o = connected[0]
                gr = self.__groupCmd
                m = self.menuPhase
                m.clear()
                for f in o.phases():
                    a = m.addAction('%i:%s' % (f.id, f.description))
                    a.setData(('SET_PHASE', f.id))
                    if o.currentPhase == f.id:
                        a.checkable = True
                        a.checked = True
                    gr.addAction(a)
                if commandAvailable('SET_PHASE'):
                    menu.addMenu(m)

                m = self.menuProgram
                m.clear()
                for p in o.programs():
                    a = m.addAction('%i:%s' % (p.id, p.description))
                    a.setData(('SET_PROG', p.id))
                    if o.currentProgram == p.id:
                        a.checkable = True
                        a.checked = True
                    gr.addAction(a)
                if hasattr(plugins,'concon') and self.__has_cnfstorage():
                    m.addSeparator()
                    if self.__has_metarpc():
                        if commandAvailable('INVOKE_StorageApp_getProg(QString)','MetaRpc') and commandAvailable('INVOKE_StorageApp_putDprog(QString,QByteArray)','MetaRpc'):
                            a = m.addAction(u'Изменить...')
                            a.connect('triggered()', self.changeProg)
                        if commandAvailable('INVOKE_StorageApp_putDprog(QString,QByteArray)','MetaRpc') and o.currentProgram>49 and o.currentProgram<54:
                            a = m.addAction(u'Отменить')
                            a.setData('MetaRpc/INVOKE_StorageApp_putDprog(QString,QByteArray)')
                            a.connect('triggered()', self.cancelProg)
                    else:
                        a = m.addAction(u'Изменить...')
                        a.connect('triggered()', self.changeProg)
                        if o.currentProgram>49 and o.currentProgram<54:
                            a = m.addAction(u'Отменить')
                            a.connect('triggered()', self.cancelProg)

                if commandAvailable('SET_PROG'):
                    menu.addMenu(m)

            no_menu_special=True
            for a in self.menuSpecial.actions():
                a.visible=commandAvailable(a.data()[0])
                if no_menu_special and a.visible:
                    no_menu_special=False
            if not no_menu_special:
                menu.addMenu(self.menuSpecial)

            #Cлужебные
            no_menu_service=True
            for a in self.menuService.actions():
                a.visible=SubMenu.checkAction(a)
                if no_menu_service and a.visible:
                    no_menu_service=False
            if not no_menu_service:
                menu.addMenu(self.menuService)
                if hasattr(self,'sgDiagramm'):
                    self.sgDiagramm.visible=(count==1 and (o.type[:6] == u'Спектр' or o.type[:7] == u'Spectr2' or o.type[:7] == u'UK_GEAR' or o.type[:8] == u'CrossRs4' or o.type[:3] == u'МФУ'))

            #Команды без подгруппы
            for a in self.__actions:
                if not SubMenu.checkAction(a):
                    continue;
                menu.addAction(a)

            if (spCount or ukgCount or sp2Count or mfuCount) and hasattr(plugins,'DownloadQueue'):  # Журнал
                d = QtCore.QDir(plugins.DownloadQueue.journalDir)
                a = self.menuJourn.actions()
                for _a in a: _a.visible=True
                for dev in spectr:
                    if d.exists('Jrn%05u.bin' % dev.id):
                        a[1].visible = True  # Докачать
                        a[2].visible = True  # Открыть
                        break
                    else:
                        a[1].visible = False  # Докачать
                        a[2].visible = False  # Открыть
                for dev in uk_gear:
                    if d.exists('Jrn%05u.bin' % dev.id):
                        a[1].visible = True  # Докачать
                        a[2].visible = True  # Открыть
                        break
                    else:
                        a[1].visible = False  # Докачать
                        a[2].visible = False  # Открыть
                for dev in spectr2:
                    if d.exists('Jrn%05u.bin' % dev.id):
                        a[1].visible = True  # Докачать
                        a[2].visible = True  # Открыть
                        break
                    else:
                        a[1].visible = False  # Докачать
                        a[2].visible = False  # Открыть

                if commandAvailable('GET_JRNL'):
                    menu.addMenu(self.menuJourn)

            #ВПУ
            if ecuEnabled:
                no_menu_ecu=True
                for a in self.menuEcu.actions():
                    a.visible=commandAvailable(a.data()[0])
                    if no_menu_ecu and a.visible:
                        no_menu_ecu=False
                if not no_menu_ecu:
                    menu.addMenu(self.menuEcu)
        else:
            if (spCount or ukgCount or sp2Count or mfuCount) and hasattr(plugins,'DownloadQueue'):  # Журнал
                d = QtCore.QDir(plugins.DownloadQueue.journalDir)
                a = self.menuJourn.actions()
                for _a in a: _a.visible=False
                for dev in spectr:
                    if d.exists('Jrn%05u.bin' % dev.id):
                        a[1].visible = True  # Открыть
                        break
                if not a[1].visible:
                    for dev in uk_gear:
                        if d.exists('Jrn%05u.bin' % dev.id):
                            a[1].visible = True  # Открыть
                            break
                    for dev in spectr2:
                        if d.exists('Jrn%05u.bin' % dev.id):
                            a[1].visible = True  # Открыть
                            break
                if a[1].visible and commandAvailable('GET_JRNL'):
                    menu.addMenu(self.menuJourn)

        if hasattr(plugins,'ControlPanelPlugin'):
            cp_enabled=True
            for cmd in ('SET_PHASE','SET_LOCAL','SET_YF','SET_OS','SET_START'):
                if not commandAvailable(cmd):
                    cp_enabled=False
                    break
            if cp_enabled:
                a = menu.addAction(u'Панель управления')
                a.connect('triggered()', self.controlPanel)

        #Пользовательские пункты
        if hasattr(self,'menuCustom'):
            for a in self.menuCustom.actions():
                a.visible=SubMenu.checkAction(a)
        if osmView.rect.contains(osmView.mapFromGlobal(QtGui.QCursor.pos())):
            act = menu.addAction(u"Показать в дереве объектов")
            act.connect('triggered()', showSelectionOnTree)
        else:
            act = menu.addAction(u"Показать на карте")
            act.connect('triggered()', showControllerOnMap)
        self.beforeMenu(self,connected)

        return SubMenu.menu(self, obj)
    
    def actionCnfFullHistory(self):
        pass
    
    def actionReadCnfHistory(self,act):
        if not self.__has_jsonrpc_cnfstorage():
            return
        for dev in self.selection:
            plugins.DownloadQueue.loadConfigFromStorage(plugins.JsonRpcClientPlugin,dev,act.text)
            break

    def __has_cnfstorage(self):
        if self.__has_metarpc():
            return True
        return self.__has_jsonrpc_cnfstorage()

    def __has_metarpc(self):
        return has_metarpc_module and hasattr(metarpc,'cnfStorage')
    
    def __has_jsonrpc_cnfstorage(self):
        if not hasattr(plugins,'JsonRpcClientPlugin'):
            return False
        return 'cnfStorage' in plugins.JsonRpcClientPlugin.connectedObjects()

    def _cancel_dev_prog(self,dev):
        if self.__has_metarpc():
            metarpc.cnfStorage.putDprog(dev.id,'<Controller/>')
        else:
            plugins.JsonRpcClientPlugin.sendCommand('cnfStorage','app.putDprog2',{'obj_id':dev.id,'xml_data':'<Controller/>'})
